import { ComponentStore } from '@ngrx/component-store';
import { MonoTypeOperatorFunction, Observable, Subscription } from 'rxjs';
export declare type StoreState<TStore extends EnhancedComponentStore> = TStore extends EnhancedComponentStore<infer TComponentState> ? TComponentState : {};
export declare type StoreStateKeys<TState extends object> = Array<keyof TState & string>;
export declare type StoreSelectors<TState extends object> = {
    [K in StoreStateKeys<TState>[number] as `${K}$`]: Observable<TState[K]>;
};
export declare type StoreUpdaters<TState extends object> = {
    [K in StoreStateKeys<TState>[number] as `set${Capitalize<K>}`]: (value: TState[K]) => void;
};
export declare type AsyncStoreUpdaters<TState extends object> = {
    [K in StoreStateKeys<TState>[number] as `set${Capitalize<K>}`]: (observableValue: Observable<TState[K]>) => Subscription;
};
export declare function getSelectors<TStore extends EnhancedComponentStore>(store: TStore): StoreSelectors<StoreState<TStore>>;
export declare function capitalize<T extends string>(str: T): Capitalize<T>;
export declare function getUpdaters<TStore extends EnhancedComponentStore>(store: TStore): StoreUpdaters<StoreState<TStore>> & AsyncStoreUpdaters<StoreState<TStore>>;
export interface EnhancedEffectReturn<ObservableType> {
    get hasRunOnce(): boolean;
    set hasRunOnce(v: boolean);
    setCleanupFn: (cleanupFn: (value: ObservableType | undefined) => void) => void;
}
export declare abstract class EnhancedComponentStore<TState extends object = any> extends ComponentStore<TState> {
    #private;
    readonly selectors: StoreSelectors<TState>;
    readonly updaters: StoreUpdaters<TState> & AsyncStoreUpdaters<TState>;
    protected constructor(state: TState);
    /**
     *
     * @deprecated use {@link tapEffect} with {@link ComponentStore.prototype.effect} instead
     */
    enhancedEffect<// This type quickly became part of effect 'API'
    ProvidedType = void, OriginType extends Observable<ProvidedType> | unknown = Observable<ProvidedType>, ObservableType = OriginType extends Observable<infer A> ? A : never, ReturnType = ProvidedType | ObservableType extends void ? () => void : (observableOrValue: ObservableType | Observable<ObservableType>) => Subscription>(generator: (origin$: OriginType) => Observable<unknown>): ReturnType & EnhancedEffectReturn<ObservableType>;
    getImperativeState(): TState;
}
/**
 * An extended `tap` operator that accepts an `effectFn` which:
 * - runs on every `next` notification from `source$`
 * - can optionally return a `cleanUp` function that
 * invokes from the 2nd `next` notification onward and on `unsubscribe` (destroyed)
 *
 *
 * @example
 * ```typescript
 * source$.pipe(
 *  tapEffect((sourceValue) = {
 *    const cb = () => {
 *      doStuff(sourceValue);
 *    };
 *    addListener('event', cb);
 *
 *    return () => {
 *      removeListener('event', cb);
 *    }
 *  })
 * )
 * ```
 */
export declare function tapEffect<T>(effectFn: (value: T, firstRun: boolean) => ((previousValue: T | undefined, isUnsubscribed: boolean) => void) | void): MonoTypeOperatorFunction<T>;
